package com.seamew.lottery.infrastructure.util;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * @Author: seamew
 * @Title: RedisUtil
 * @CreateTime: 2023年03月03日 10:26:00
 * @Description: Redis 工具类
 * @Version: 1.0
 */
@Component
public class RedisUtil {
    @Autowired
    private StringRedisTemplate redisTemplate;

    /**
     * 递增
     *
     * @param key   键
     * @param delta 要增加几(大于0)
     * @return 执行结果
     */
    public Long incr(String key, long delta) {
        if (delta < 0) {
            throw new RuntimeException("递增因子必须大于0");
        }
        return redisTemplate.opsForValue().increment(key, delta);
    }

    /**
     * 递减
     *
     * @param key   键
     * @param delta 要减少几(小于0)
     * @return 执行结果
     */
    public Long decr(String key, long delta) {
        if (delta < 0) {
            throw new RuntimeException("递减因子必须大于0");
        }
        return redisTemplate.opsForValue().increment(key, -delta);
    }

    /**
     * 分布式锁
     *
     * @param key            锁住的key
     * @param lockExpireMils 锁住的时长。如果超时未解锁，视为加锁线程死亡，其他线程可夺取锁
     * @return 执行结果
     */
    public Boolean setNx(String key, Long lockExpireMils) {
        return (Boolean) redisTemplate.execute((RedisCallback<?>) connection -> {
            // 获取时间毫秒值
            long expireAt = System.currentTimeMillis() + lockExpireMils + 1;
            //获取锁
            Boolean acquire = connection.setNX(key.getBytes(), String.valueOf(expireAt).getBytes());
            // 增加超时处理逻辑
            if (Boolean.TRUE.equals(acquire)) {
                return true;
            }
            byte[] value = connection.get(key.getBytes());

            if (Objects.nonNull(value) && value.length > 0) {
                // 将获取到的值转为long类型，便于进行比较
                long expireTime = Long.parseLong(new String(value));
                // 如果超时，重新加锁，防止死锁
                if (expireTime < System.currentTimeMillis()) {
                    byte[] oldValue = connection.getSet(key.getBytes(), String.valueOf(System.currentTimeMillis() + lockExpireMils + 1).getBytes());
                    return Long.parseLong(new String(oldValue)) < System.currentTimeMillis();
                }
            }
            return false;
        });
    }

    /**
     * 删除缓存
     *
     * @param key 一个或多个
     */
    public void del(String... key) {
        if (key != null && key.length > 0) {
            if (key.length == 1) {
                redisTemplate.delete(key[0]);
            } else {
                redisTemplate.delete(Arrays.asList(key));
            }
        }
    }

    /**
     * 普通缓存获取
     *
     * @param key 键
     * @return 值
     */
    public String get(String key) {
        return redisTemplate.opsForValue().get(key);
    }

    /**
     * 普通缓存放入
     *
     * @param key   键
     * @param value 值
     * @return true成功 false失败
     */
    public boolean set(String key, Object value) {
        try {
            redisTemplate.opsForValue().set(key, String.valueOf(value));
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    /**
     * 指定缓存失效时间
     *
     * @param key  键
     * @param time 时间(秒)
     * @return 执行结果
     */
    public boolean expire(String key, long time) {
        try {
            if (time > 0) {
                redisTemplate.expire(key, time, TimeUnit.SECONDS);
            }
            return true;
        } catch (Exception e) {
            return false;
        }
    }
}
